/*
    graph library based on array lists
    (#vertices and #edges can not be increased after the initialization
            12/Feb/2002   by Takeaki Uno  e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, please
    refer the latestt code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users. */


#ifndef _agraph_c_
#define _agraph_c_

#include"agraph.h"
#include"alist.c"
#include"stdlib2.c"

/* Commnet out if no error notification is needed */
//#define ERROR_CHECK

AGRAPH INIT_AGRAPH = {TYPE_AGRAPH,NULL,0,0,0,0,0,0,0,{TYPE_MALIST,0,0,0,0,0,0},{TYPE_MALIST,0,0,0,0,0,0},{TYPE_MALIST,0,0,0,0,0,0},0,NULL,NULL,NULL};

/* Initialization for graph object  */
void AGRAPH_alloc (AGRAPH *G, ALIST_ID node_num, ALIST_ID edge_num, ALIST_ID arc_num){
  ALIST_ID i;
  G->node_end = node_num;  // maximum number of vertices
  G->edge_end = edge_num;  // maximum number of edges
  G->arc_end = arc_num;    // maximum number of directed arcs
  if ( edge_num>0 ){
    MALIST_alloc (&(G->edge), edge_num*2, node_num+1); //edge list for each vertex
    for (i=0 ; i<G->edge_end ; i++)  // insert all edges to "deleted edge list"
        MALIST_ins_tail (&(G->edge), G->node_end, i, 0);
  }
  if ( arc_num>0 ){
    MALIST_alloc (&(G->in), arc_num, node_num+1); // in-arc list for each vertex
    MALIST_alloc (&(G->out), arc_num, node_num); // out-arc list for each vertex
    for (i=0 ; i<G->arc_end ; i++)  // insert all arcs to "deleted arc list"
        MALIST_ins_tail (&(G->in), G->node_end, i, 0);
  }
  G->node1_num = 0;    // in bipartite case, {0,...,node1_num-1} is the vertex set 1, and the rest is the other
  G->node_w = (WEIGHT *)NULL; // vertex, edge, arc weight arrays are set to NULL
  G->edge_w = (WEIGHT *)NULL;
  G->arc_w = (WEIGHT *)NULL;
}

/* Allocate memory for weight arrays */
void AGRAPH_alloc_weight (AGRAPH *G){
  malloc2 (G->node_w, G->node_end, EXIT);
  malloc2 (G->edge_w, G->edge_end, EXIT);
  malloc2 (G->arc_w, G->arc_end, EXIT);
}

/* Copy graph G to graph G2. Underconstruction */
void AGRAPH_cpy (AGRAPH *G2, AGRAPH *G){}

/* Free graph object  */
void AGRAPH_end (AGRAPH *G){
  if ( G->edge_end > 0 ) MALIST_end (&(G->edge));
  if ( G->arc_end > 0 ){
    MALIST_end (&(G->in));
    MALIST_end (&(G->out));
  }
  mfree (G->node_w, G->edge_w, G->arc_w);
  *G = INIT_AGRAPH;
}

/*  Set e to an edge connecting vertex u and vertex v. If e is already set, 
  there might occur error. e, u, and v are given by IDs.
  Supporting range checking. */
void AGRAPH_edge_init (AGRAPH *G, ALIST_ID e, ALIST_ID u, ALIST_ID v, int undo){
#ifdef ERROR_CHECK
  if ( e<0 || e >= G->edge_end ) error_num ("AGRAPH_edge_init: edge ID is out of range", e, EXIT);
#endif
  MALIST_ins_tail (&(G->edge), u, e, undo);
  MALIST_ins_tail (&(G->edge), v, e+G->edge_end, undo);
  G->edge_num ++;
  if ( undo ) ALISTundo_ins (U_AGRAPH_edge_num, (void *)G, -1, 0);
}

/* Get e from the deleted edge list, and set e to an edge connecting vertex
  u and vertex v. Return the ID of e. Supporting range checking.  */
ALIST_ID AGRAPH_edge_mk (AGRAPH *G, ALIST_ID u, ALIST_ID v, int undo ){
  ALIST_ID e = MALIST_rm_head (&G->edge, G->edge.list_end-1, undo);
  if ( !MALIST_IS_ELM(G->edge,e) )
      error ("AGRAPH_edge_mk: edge array overflow.\n", EXIT0);
  AGRAPH_edge_init (G, e, u, v, undo);
  return (e);
}

/*  The remove, delete, and recover of edge e. e can be given by working ID.
    Error occurs if delete/remove an edge which has not been set.
    Supporting range checking */
void AGRAPH_edge_out (AGRAPH *G, ALIST_ID e, int undo){
#ifdef ERROR_CHECK
  if ( e < 0 || e >= G->edge.end )
      error_num ("AGRAPH_edge_out: removing edge of out of range\n", e, EXIT);
#endif
  e = e % G->edge_end; 
  MALIST_rm (&(G->edge), e + G->edge_end, undo);
  MALIST_rm (&(G->edge), e, undo);
  G->edge_num--;
  if ( undo ) ALISTundo_ins (U_AGRAPH_edge_num, (void *)G, 1, 0);
}
void AGRAPH_edge_rm (AGRAPH *G, ALIST_ID e, int undo ){
  AGRAPH_edge_out (G, e, undo);
  e = e % G->edge_end;
  MALIST_ins_head (&(G->edge), G->edge.list_end-1, e, undo);
}
void AGRAPH_edge_recov (AGRAPH *G, ALIST_ID e, int undo){
#ifdef ERROR_CHECK
  if ( e < 0 || e >= G->edge.end )
      error_num ("AGRAPH_edge_recov: recovering edge of out of range.%d\n", e, EXIT);
#endif
  e = e % G->edge_end;
  MALIST_recov (&(G->edge), e, undo);
  MALIST_recov (&(G->edge), e + G->edge_end, undo);
  G->edge_num++;
  if ( undo ) ALISTundo_ins (U_AGRAPH_edge_num, (void *)G, -1, 0);
}

/* Set a to an arc from vertex u to vertex v. Error occurs if delete/remove
 an arc which has not been set. a, u, and v are given by IDs.
 Supporting range checking. */
void AGRAPH_arc_init (AGRAPH *G, ALIST_ID a, ALIST_ID from, ALIST_ID to, int undo){
#ifdef ERROR_CHECK
  if ( a < 0 || a >= G->in.end ) error_num ("AGRAPH_arc_init: arc is out of range", a, EXIT);
#endif
  MALIST_ins_tail (&(G->in), to, a, undo);
  MALIST_ins_tail (&(G->out), from, a, undo);
  G->arc_num++;
  if ( undo ) ALISTundo_ins (U_AGRAPH_arc_num, (void *)G, -1, 0);
}

/* Get an arc a from the deleted arc list, and set a to an arc from vertex
 u to vertex v. Supporting range checking. */
ALIST_ID AGRAPH_arc_mk (AGRAPH *G, ALIST_ID from, ALIST_ID to, int undo){
  ALIST_ID a = MALIST_rm_head (&(G->in), G->in.list_end-1, undo);
  if ( !MALIST_IS_ELM(G->in,a) )
      error ("AGRAPH_arc_mk: the arc array is overflow", EXIT0);
  AGRAPH_arc_init (G, a, from, to, undo);
  return (a);
}

/* Remove/delete/recover of an arc. Error occurs if delete/remove an arc 
 which has not been set. Supporting range checking. */
void AGRAPH_arc_out (AGRAPH *G, ALIST_ID a, int undo){
#ifdef ERROR_CHECK
  if ( a < 0 || a >= G->in.end ) error_num ("AGRAPH_arc_out: arc is out of range", a, EXIT);
#endif
  MALIST_rm (&(G->in), a, undo);
  MALIST_rm (&(G->out), a, undo);
  G->arc_num--;
  if ( undo ) ALISTundo_ins (U_AGRAPH_arc_num, (void *)G, 1, 0);
}
void AGRAPH_arc_rm (AGRAPH *G, ALIST_ID a, int undo ){
  AGRAPH_arc_out (G, a, undo);
  MALIST_ins_head (&(G->in), G->in.list_end-1, a, undo);
}
void AGRAPH_arc_recov (AGRAPH *G, ALIST_ID a, int undo){
#ifdef ERROR_CHECK
  if ( a < 0 || a >= G->in.end ) error_num ("AGRAPH_arc_out: arc is out of range", a, EXIT);
#endif
  MALIST_recov (&(G->in), a, undo);
  MALIST_recov (&(G->out), a, undo);
  G->arc_num++;
  if ( undo ) ALISTundo_ins (U_AGRAPH_arc_num, (void *)G, -1, 0);
}


/* Delete all arcs and edges incident to vertex v */
void AGRAPH_node_out (AGRAPH *G, ALIST_ID v, int undo){
  ALIST_ID e;
#ifdef ERROR_CHECK
  if ( v < 0 || v >= G->node_end ) error_num ("AGRAPH_node_out: vertex is out of range", v, EXIT);
#endif
  if ( G->edge_end>0 ){
    MALIST_DO_FORWARD (G->edge, v, e )
      if ( G->edge.prv[AGRAPH_OPO(*G,e)] >= 0 ) MALIST_rm (&(G->edge), AGRAPH_OPO(*G,e), undo);
  }
  if ( G->arc_end>0 ){
    MALIST_DO_FORWARD (G->in, v, e )
        MALIST_rm (&(G->out), e, undo);
    MALIST_DO_FORWARD (G->out, v, e )
        MALIST_rm (&(G->in), e, undo);
  }
}
/* Remove a node from the graph */
void AGRAPH_node_rm (AGRAPH *G, ALIST_ID v, int undo){
  ALIST_ID e;
#ifdef ERROR_CHECK
  if ( v < 0 || v >= G->node_end ) error_num ("AGRAPH_node_rm: vertex is out of range", v, EXIT);
#endif
  if ( G->edge_end>0 ){
    MALIST_DO_FORWARD (G->edge, v, e){
      AGRAPH_edge_rm (G, e, undo);
      G->edge_num--;
      if ( undo ) ALISTundo_ins (U_AGRAPH_edge_num, (void *)G, 1, 0);
    }
  }
  if ( G->arc_end>0 ){
    MALIST_DO_FORWARD (G->in, v, e){
      AGRAPH_arc_rm (G, e, undo);
      G->arc_num--;
      if ( undo ) ALISTundo_ins (U_AGRAPH_arc_num, (void *)G, 1, 0);
    }
    MALIST_DO_FORWARD (G->out, v, e){
      AGRAPH_arc_rm (G, e, undo);
      G->arc_num--;
      if ( undo ) ALISTundo_ins (U_AGRAPH_arc_num, (void *)G, 1, 0);
    }
  }
}
/* Recover a removed node */
void AGRAPH_node_recov (AGRAPH *G, ALIST_ID v, int undo){
  ALIST_ID e;
#ifdef ERROR_CHECK
  if ( v < 0 || v >= G->node_end ) error_num ("AGRAPH_node_out: vertex is out of range", v, EXIT);
#endif
  if ( G->edge_end>0 ){
    MALIST_DO_BACKWARD (G->edge, v, e){
      MALIST_recov (&(G->edge), AGRAPH_OPO(*G,e), undo);
      G->edge_num++;
      if ( undo ) ALISTundo_ins (U_AGRAPH_edge_num, (void *)G, -1, 0);
    }
  }
  if ( G->arc_end>0 ){
    MALIST_DO_BACKWARD (G->in, v, e){
      MALIST_recov (&(G->out), e, undo);
      G->arc_num++;
      if ( undo ) ALISTundo_ins (U_AGRAPH_arc_num, (void *)G, -1, 0);
    }
    MALIST_DO_BACKWARD (G->out, v, e){
      MALIST_recov (&(G->in), e, undo);
      G->arc_num++;
      if ( undo ) ALISTundo_ins (U_AGRAPH_arc_num, (void *)G, -1, 0);
    }
  }
}

/* A Function to return the maximum degree of the graph */
ALIST_ID AGRAPH_max_degree (AGRAPH *G){
  ALIST_ID i, m=0;
  FLOOP (i, 0, G->node_end){
    if ( G->edge.num ) ENMAX (m, G->edge.num[i]);
    if ( G->in.num ) ENMAX (m, G->in.num[i]+G->out.num[i]);
  }
  return (m);
}

/* make an array of vertices sorted in the order of degrees.
   Increasing order if flag==1, and decreasing order if flag ==-1 */
/* return the inverse permutation (original => internal) */
PERM *AGRAPH_degree_sort (AGRAPH *G, int flag){
  PERM *perm=0, *perm_=0;
  ALIST_ID i;
  malloc2 (perm, G->node_end, EXIT0);
  perm_ = qsort_perm_ALIST_ID (&(G->edge.num[G->node1_num]), G->node_end-G->node1_num, flag);
if ( ERROR_MES ) goto ERR;
  FLOOP (i, 0, G->node_end - G->node1_num) perm[i+G->node1_num] = perm_[i]+G->node1_num;
  free2 ( perm_ );
  if ( G->node1_num > 0 ){
    perm_ = qsort_perm_ALIST_ID (G->edge.num, G->node1_num, flag);
if ( ERROR_MES ) goto ERR;
    memcpy ( perm, perm_, G->node1_num*sizeof(ALIST_ID) );
  }
  return (perm);
  ERR:;
  mfree (perm, perm_);
  EXIT0;
}

/* For each vertex, sort the edges incident to the vertex by the IDs of
   the vertices they are adjacent to. */
void AGRAPH_sort_incident_edges (AGRAPH *G){
  ALIST_ID *ff, i;
  malloc2 (ff, G->edge.end, EXIT);
  FLOOP (i, 0, G->edge.end) ff[i] = AGRAPH_INC(*G, i);
  FLOOP (i, 0, G->node_end) MALIST_sort_odr (&(G->edge), i, 0, ff, 0);
  free (ff);
}

/* If G is a bipartite graph such that the vertex set one is from 0 to n1-1, 
 and the vertex set two is from n1 to node_num-1. Then, find such n1.
 if the graph is not in the form, return -1. */
ALIST_ID AGRAPH_find_node1_num (AGRAPH *G){
  ALIST_ID i=0, m=G->node_end, u, e;
    /* from vertex 0 to n, one-by-one, find the minimum ID vertex in
      its neighbor, and set m to it if m is larger than it. */
  while ( i < m ){ /* if i>=m, there is an edge connecting m and the vertex in
   0 to m-1. Thus, stop at i=m-1, and 0,...,i has no edge among them. */
    MALIST_DO_FORWARD (G->edge, i, e){
      u = AGRAPH_INC(*G, e);
      if ( u < m ) m = u;
    }
    MALIST_DO_FORWARD (G->in, i, e){
      u = AGRAPH_FROM(*G, e);
      if ( u < m ) m = u;
    }
    MALIST_DO_FORWARD (G->out, i, e){
      u = AGRAPH_TO(*G, e);
      if ( u < m ) m = u;
    }
    i++;
  }
    /* Check whether there is an edge connecting vertices in m,...,node_num.
    If exist, set i=-2, thereby returned value will be -1. */
  BLOOP (i, G->node_num, m){
    MALIST_DO_FORWARD (G->edge, i, e) if ( AGRAPH_INC(*G, e) >= m ) i=-2;
    MALIST_DO_FORWARD (G->in, i, e) if ( AGRAPH_INC(*G, e) >= m ) i=-2;
    MALIST_DO_FORWARD (G->out, i, e) if ( AGRAPH_INC(*G, e) >= m ) i=-2;
  }
  return (i+1);
}

/* Return -1 if edge (v,u) does not exist. If exist, then return its
 working ID of (v->u) */
ALIST_ID AGRAPH_is_edge (AGRAPH *G, ALIST_ID v, ALIST_ID u){
  ALIST_ID e;
  MALIST_DO_FORWARD (G->edge, v, e) if ( AGRAPH_INC(*G,e) == u ) return (e);
  return (-1);
}

/* Check whether graph G1 is isomorphic to G2 by permutating vertex set of 
 G1 by p. If they are isomorphic, return 0 */
int AGRAPH_equal_perm (AGRAPH *G1, AGRAPH *G2, ALIST_ID *p){
  ALIST_ID v, e;
  FLOOP (v, 0, G1->node_end)
    MALIST_DO_FORWARD (G1->edge, v, e)
      if ( AGRAPH_is_edge (G2, p[v], p[AGRAPH_INC(*G1,e)]) == 0 ) return (0);
  return (1);
}

/*  print graph by the IDs of vertices, edges and arcs */
void AGRAPH_print (AGRAPH *G){
  ALIST_ID i, j;
  printf ("#node " ALIST_IDF " ,#edge " ALIST_IDF " ,#arc " ALIST_IDF "\n", G->node_end, G->edge_num, G->arc_num);
  FLOOP (i, 0, G->node_num){
    printf ("NODE " ALIST_IDF " ", i);
    if ( G->node_w ) printf ("(" WEIGHTF ")", G->node_w[i]);
    printf (" >>\n");
    if ( G->edge.num && G->edge.num[i] > 0 ){
      printf ("    edge      : ");
      MALIST_DO_FORWARD (G->edge, i, j){
        printf ("(e" ALIST_IDF ":v" ALIST_IDF, AGRAPH_EDGE (*G, j), AGRAPH_INC(*G, j));
        if ( G->edge_w ) printf ("[" WEIGHTF "]", G->edge_w[AGRAPH_EDGE (*G,j)]);
        printf ("),");
      }
      printf ("\n");
    }
    if ( G->in.num && G->in.num[i] > 0 ){
      printf ("    arc from : ");
      MALIST_DO_FORWARD (G->in, i, j){
        printf ("(a" ALIST_IDF ":v" ALIST_IDF, j, AGRAPH_FROM(*G, j));
        if ( G->arc_w ) printf ("[" WEIGHTF "]", G->edge_w[j]);
        printf ("),");
      }
      printf ("\n");
    }
    if ( G->out.num && G->out.num[i] > 0 ){
      printf ("    arc to   : ");
      MALIST_DO_FORWARD (G->out, i, j){
        printf ("(a" ALIST_IDF ":v" ALIST_IDF, j, AGRAPH_TO( *G, j  ));
        if ( G->arc_w ) printf ("[" WEIGHTF "]", G->edge_w[j]);
        printf ("),");
      }
      printf ("\n");
    }
  }
}

/* replace multiple edges and multiple arcs by single one */
void AGRAPH_simple (AGRAPH *G){
  ALIST_ID v, e, maxd, *p, i;
  maxd = AGRAPH_max_degree(G)*2;
  malloc2 (p, maxd, EXIT);
  FLOOP (v, 0, G->node_end){
    i = 0;
    MALIST_DO_FORWARD (G->edge, v, e){
      p[i] = AGRAPH_INC(*G, e);
      p[i+1] = e;
      i += 2;
    }
    qsort_ALIST_ID (p, i/2, sizeof(ALIST_ID)*2);
    for (i-=2 ; i>0 ; i-=2)
        if ( p[i] == p[i-2] ) AGRAPH_edge_rm (G, p[i+1], 0);
  }
  free (p);
}

/* routines for generating random graphs.
  flag&1 = 0 => undirected, o.w. => directed.
  flag&2 = 0 => without selfloops, o.w., with.
  flag&4 = 0 => two vertices will not be connected by arcs of both direction,
       o.w., can be.
  flag&8 = 0 => then distribution is uniform. o.w., each vertex is connected
   to vertices with IDs of +- its ID.
   All vertices will be used, from 0 to node_end */


/* generating randam graph part 1. flag%16 ==0 => the probability of the 
 existence of each edge is per/1000, o.w., #edges is just "per" */
/* node1_num >0 => generate bipartite graph */
void AGRAPH_mk_rand_graph1 (AGRAPH *G, ALIST_ID per, int flag){
  ALIST_ID i, j, c, f, a, b, r;
  
  /* Initialization. According to parameters, set loop counter variables, and 
    amount of increase of variable for random number. */
  if ( flag&16 ){
    f = 1;
    if ( G->node1_num>0 ) c = G->node1_num * (G->node_end - G->node1_num);
    else c = (G->node_end-1) * G->node_end/2;
    if ( (flag & 4) && (flag & 1) ) c *= 2;
    if ( flag & 2 ) c += G->node_end;
  } else {
    c = 1000;
    f = 0;
  }

  /* main loop */
  FLOOP (i, 0, G->node1_num>0?G->node1_num:(G->node_end-1+(flag&8)/8)){  /* Loop for vertices in vertex set one if G is bipartite, o.w., loop for all vertices. */
    a = G->node1_num>0?G->node1_num+(flag&8?i:0):(i+1-(flag&2)/2);
    FLOOP (j, a, flag&8?a+per:G->node_end){  /* Loop for vertex set two if G is bipartite, from i to the end if with selfloop, o.w., from i+1 to end. */
      b = (j-G->node1_num)%(G->node_end-G->node1_num)+G->node1_num;
      r = rand()%c;
      if ( (!(flag&8) && r < per) || ((flag&8) && (r&1)) ){  /* if random number is less than threshold */
        per -= f;    /* if #edges is given, decrease #edges to be generated */
        if ( flag & 1 )
          if ( (flag & 4) || rand()%2 ) AGRAPH_arc_mk (G, i, b, 0); /* if both direction arcs are allowed, then fix the direction to one, o.w., choose the direction romdomly */ 
            else AGRAPH_arc_mk (G, b, i, 0);
        else AGRAPH_edge_mk (G, i, b, 0);
      }
      c -= f;   /* if #edges is given, decrease the counter */
      if ( (flag & 4) && (flag & 1) ){  /* if both direction arcs are allowed, generate arcs of the other directions */
        if ( rand()%c < per ){
          per -= f;
          AGRAPH_arc_mk (G, b, i, 0);
        }
        c -= f;   /* if #edges is given, decrease the counter */
      }
    }
  }
  G->node_num = G->node_end;
}


/* A subroutine used by mk_rand_graph2, for setting edges. */
/* the direction is given by ff, ff= 0=> (i,v), o.w., (v,i) */
void AGRAH_mk_rand_graph2_iter (AGRAPH *G, ALIST_ID *f, ALIST_ID i, int flag, int ff){
  ALIST_ID v=0, c=0;
  do {
    if ( G->node1_num>0 ) /* generate an edge incident to i, randomly, deal with bipartite and normal cases independently */
      if ( i<G->node1_num )
        v = rand()%(G->node_end-G->node1_num)+G->node1_num;
      else  v = rand()%G->node1_num;
    else v = rand()%G->node_end;
    c++;
  } while ( f[v] == i && c<10000); /* f[v] is the flag to avoid multiple edges. */
  if ( c<10000 ){ /* if the edge is not multiple */
    f[v] = i;
    if ( flag & 1 )
      if ( ff ) AGRAPH_arc_mk (G, v, i, 0);
      else AGRAPH_arc_mk (G, i, v, 0);
    else AGRAPH_edge_mk (G, i, v, 0);
  }
}

/* random graph generator 2, the degree of each vertex is roughly close 
 to the specified "deg", but it works well only in the case that n is 
  large, such as over 100. #edges never exceed deg*node_end/2 */
/* node1_num >0 => generate bipartite graph */
void AGRAPH_mk_rand_graph2 (AGRAPH *G, ALIST_ID deg, int flag){
  ALIST_ID i, j, *f, e;
  malloc2 (f, G->node_end, EXIT);
  ARY_FILL (f, 0, G->node_end, -1);

  if ( flag & 1 ){ /* generate arc of one-direction, previously, in directed case */
    FLOOP (i, 0, G->node_end){
      if ( (flag & 4) == 0)
          MALIST_DO_FORWARD (G->out, i, e) f[AGRAPH_TO(*G,e)] = i;
      f[i] = i;
      FLOOP (j, 0, deg/4) /* for each vertex, make deg/2 out-going/in-coming arcs, thus loop is executed deg/4 times */
        AGRAH_mk_rand_graph2_iter (G, f, i, flag, 1);
    }
  ARY_FILL (f, 0, G->node_end, -1);
  }

  /* main loop. To avoid multiple edges, or arcs of both directed between two vertices, mark f[v] and call subroutine to generate edge */
  FLOOP (i, 0, G->node_end){
    if ( flag&1 ){
      if ( (flag & 4) == 0)
          MALIST_DO_FORWARD (G->in, i, e ) f[AGRAPH_FROM(*G,e)] = i;
      MALIST_DO_FORWARD (G->out, i, e ) f[AGRAPH_TO(*G,e)] = i;
    } else MALIST_DO_FORWARD (G->edge, i, e ) f[AGRAPH_INC(*G, e)] = i;
    if ( (flag&2) == 0 ) f[i] = i;
    FLOOP (j, 0, deg/(2+(flag&1)*2))
        AGRAH_mk_rand_graph2_iter (G, f, i, flag, 0);
  }
  free (f);
  G->node_num = G->node_end;
}


/* Write the graph to file. Edges, arcs, and nodes from 0 to node_num/edge_num/arc_num are written to file. Parameters are
  (graph) (file name) (flag)
   AGRAPH_READ_NODEW 512 // read node weight
   AGRAPH_READ_EDGEW 1024 // read edge weight

  format of file:(including notifications to make input file)
   
  the ith row corresponds to node i-1, and
    ID list of nodes adjacent to i, and having ID > i, for undirected graph
    ID list of nodes adjacent to i by out-going arc of i, for directed graph
   Separator is ",", but graph load routine accepts any letter for 
    separator but not a number.
   If the graph has both edges and arcs, write them in two lines separately,
    so a node then uses two lines, and #nodes = #lines/2.
  
    ==  Notifications to make input file ==
   Notice that if 0th line has node 2, and the 2nd line has 0, then there
    will be multiple edge (0,2) and (2,0).
   The read routine does not make error with multiple edges, it is allowed.

   The ID of nodes begin from 0. After reading graph, node_num is set to
    node_end.

   Input file example, without weights, E={(0,1),(0,2),(1,1),(1,3),(2,3)}
===========
   1,2
   1 3
   3
   
   [EOF]
=========
   Nodes are 0,1, and 2, both edges and arcs exist, with node/edge/arc weights)
   5000,1,30
   0,50,1,20,
   100,1,3
   2,20
   200
   
   [EOF]
=======
   where node weights are 5000, 100, 200, and edges and their weights are
    (0,1),30,   (1,1),3
    arcs and their weights are (0,0),50,   (0,1), 20,   (1,2), 20

    In the case of bipartite graph, write the adjacent-node lists only for 
     the node in node set one.
     
    
*/
void AGRAPH_save (AGRAPH *G, char *fname){
  ALIST_ID i, e, u,f;
  FILE *fp;

    /* open file */
  fopen2 (fp, fname, "w", exit(1));
  FLOOP (i, 0, G->node_num){
    f = 0;
    if ( G->flag & LOAD_NODEW ){ f=1; fprintf (fp, WEIGHTF, G->node_w[i]);} /* write node weight */
    if ( G->edge_num > 0){
      MALIST_DO_FORWARD (G->edge, i, e ){
        u = AGRAPH_INC(*G, e);
        if ( u>i || (u == i && e < G->edge_end) ){ /* avoid duplicated write for one edge */
          if ( f ) fprintf (fp, ",");
          fprintf (fp, ALIST_IDF, u);
          if ( G->flag & LOAD_EDGEW ) fprintf (fp, "," WEIGHTF, G->edge_w[AGRAPH_EDGE(*G,e)]); /* write edge weight */
          f = 1; 
	}
      }
    }
    if ( G->arc_num > 0){ /* write arcs */
      MALIST_DO_FORWARD (G->out, i, e){
        if ( f ) fprintf (fp, ",");
        fprintf ( fp, ALIST_IDF, AGRAPH_TO(*G,e));
        if ( G->flag & LOAD_EDGEW ) fprintf (fp, "," WEIGHTF, G->arc_w[e]); /* write arc weight */
        f = 1; 
      }
    }
    fprintf (fp, "\n");
  }
  fclose (fp);
}

/* graph save routine with node permutation */
void AGRAPH_save_perm (AGRAPH *G, char *fname, int flag, ALIST_ID *p, ALIST_ID *q){
/* under construction */
}


/* graph load routine. Allocate memory as much as the size of input file.
   parameters are, 
   (graph) (file name) (flag)
 LOAD_EDGE  // read undirected edge from file
 LOAD_ARC  // read directed arc from file
 AGRAPH_BIPARTITE // load bipartite graph
 LOAD_NODEW // read node weight
 LOAD_EDGEW // read edge weight
*/
/* In the bipartite case, even if the IDs of node set 2 begin from 0, i.e.,
   overlaps with node 1, the routine automatically correct them. */
/* Directed bipartite graph, all arcs are considered to be from node set 1
 to node set 2. If both directions exist, read as a general graph, and set
  node1_num later in some way. */
/* The routine compares the maximum node index and #lines, and set #node
  to the larger one. However, if node weight exists, weights will be included 
  in the candidates of maximum index, thus in this case we fix #node := #lines.
  In the case of bipartite graph, the routine compares, but the weights of 
   non-existing lines will be -1. */
void AGRAPH_load (AGRAPH *G){
  ALIST_ID i, x, k, base=0, node_end;
  FILE2 fp = INIT_FILE2;
  FILE_COUNT C;

      /* open the file, and read properties, such as #nodes, #edges */
//  edge_num = FILE2_count (filename, &node_num, &max_node, &min_node, NULL);
  FILE2_open (fp, G->fname, "r", EXIT);
  C = FILE2_count (&fp, 0, 0, 0, 0, 0, 0);
if ( ERROR_MES ) return;
  if ( (G->flag & LOAD_BIPARTITE) && C.clm_btm<C.rows ) base = C.rows;
  if ( G->flag & (LOAD_NODEW + LOAD_EDGEW) ) node_end = C.rows+base;
    else node_end = MAX(C.rows, C.clms+1+base);
  if ( G->flag & LOAD_EDGE ){
    if ( G->flag & LOAD_ARC ) AGRAPH_alloc (G, node_end/2, C.eles, C.eles);
    else AGRAPH_alloc (G, node_end, C.eles, 0);
  } else AGRAPH_alloc (G, node_end, 0, C.eles);
  G->node_num = G->node_end;
  if ( G->flag & LOAD_BIPARTITE ) G->node1_num = C.rows;
  if ( G->flag & (LOAD_NODEW + LOAD_EDGEW) ) AGRAPH_alloc_weight (G); // allocate array for weights
if ( ERROR_MES ) return;

  FILE2_reset (&fp);
  FLOOP (i, 0, node_end){
      /* read process for edges */
    if ( G->flag & LOAD_NODEW ) G->node_w[i] = (WEIGHT)FILE2_read_int (&fp); // read node weight
    if ( G->flag & LOAD_EDGE ){
      do {
        x = (ALIST_ID)FILE2_read_int (&fp) + base;
        if ( FILE_err&4 ) break;
        k = AGRAPH_edge_mk (G, i, x, 0);
        if ( G->flag & LOAD_EDGEW ){
          G->edge_w[k] = (WEIGHT)FILE2_read_double (&fp); // read edge weights
          if ( FILE_err&4 ) error ("AGRAPH_load: edge weight is absent", goto END);
        }
      } while ( (FILE_err&3) == 0 );
    }
    if ( G->flag & LOAD_ARC ){
        /* read process for arcs */
      do {
        x = (ALIST_ID)FILE2_read_int (&fp) + base;
        if ( FILE_err&4 ) break;
        k = AGRAPH_arc_mk (G, i, x, 0);
        if ( G->flag & LOAD_EDGEW ){
          G->arc_w[k] = (WEIGHT)FILE2_read_double (&fp); /* read edge weight */
          if ( FILE_err&4 ) error ("AGRAPH_load: arc weight is absent", goto END);
        }
      } while ( (FILE_err&3)==0 );
    }
  }
  END:;
  FILE2_close (&fp);
  if ( ERROR_MES ){ free2 (G->edge_w); free2 (G->node_w); free2 (G->arc_w); EXIT;}
  else print_mes (G, "agraph: %s ,#nodes %d ,#edges %zd ,#arcs %zd\n", G->fname, G->node_num, G->edge_num,  G->arc_num);

}


/*
main (){
  AGRAPH G;
  int i, j;

  while (1){
    AGRAPH_init ( &G, 10, 100, 200);
    AGRAPH_del (&G);
  }
 
  G.node_num = 10;
  for ( i=0 ; i<5 ; i++ ){
    for ( j=i ; j<5 ; j=j+2 ){
      AGRAPH_edge_mk ( &G, i, j);
    }
  }
  AGRAPH_print ( &G);
  for ( i=3 ; i<8 ; i++ ){
    AGRAPH_edge_rm ( &G, i);
  }
  AGRAPH_print ( &G);
  for ( i=0 ; i<5 ; i++ ){
    for ( j=i ; j<5 ; j=j+2 ){
      AGRAPH_edge_mk ( &G, i, j);
    }
  }
  AGRAPH_print ( &G);
  for ( i=5 ; i<10 ; i++ ){
    for ( j=i ; j<10 ; j=j+2 ){
      AGRAPH_arc_mk ( &G, i, j);
    }
  }
  AGRAPH_print ( &G);
  for ( i=3 ; i<8 ; i++ ){
    AGRAPH_arc_rm ( &G, i);
  }
  AGRAPH_print ( &G);
  for ( i=5 ; i<10 ; i++ ){
    for ( j=i ; j<10 ; j=j+2 ){
      AGRAPH_arc_mk ( &G, i, j);
    }
  }
  AGRAPH_print ( &G);
  

}


*/
/*
main ( int argc, char *argv[] ){
  AGRAPH G;
  int v, per, flag;
  if ( argc < 6 ) error ( "randgrp: size per flag b filename");
  v = atoi(argv[1]);
  per = atoi(argv[2]);
  flag = atoi(argv[3]);
  if ( flag & 8 ) AGRAPH_init ( &G, v, v*per*2, 0);
  else AGRAPH_init ( &G, v, v*v*(per+300)/1000, 0);
  G.node1_num = atoi(argv[4]);
  AGRAPH_mk_rand_graph1 ( &G, per, flag);
  AGRAPH_save ( &G, argv[5], 0, 0);
}
*/


/*
// graph equal no main desu
main ( int argc, char *argv[] ){
  AGRAPH G1, G2;
  int *p, i;
  if ( argc < 4 ) error ( "grpcmp: filename1 filename2 perm");
  AGRAPH_load ( &G1, argv[1]);
  AGRAPH_load ( &G2, argv[2]);
  p = fscan_int_array( argv[3]);
  i = AGRAPH_equal_perm ( &G1, &G2, p);
  printf ("%d\n", i);
  free ( p);
}
*/

/*
main ( int argc, char *argv[] ){
  AGRAPH G;
  AGRAPH_load ( &G, "G2.GRP", 0, 1, 0, 1, 0);
  AGRAPH_print ( &G);
  AGRAPH_save ( &G, "tmp.grp", 0, 1);
}
*/

#endif
